package org.example;

import java.util.*;
import java.util.stream.Collectors;

public class Main {
    public static void main(String[] args) {
        System.out.println("Hello world!");
        List<Employee> employees = new ArrayList<Employee>();
        employees.add(new Employee(101, "Ashwini", "IT",25,20000.0,Arrays.asList("java","python")));
        employees.add(new Employee(102, "Amulu", "CSE",30,200000.0,Arrays.asList(".net","python")));
        employees.add(new Employee(101, "Anu", "IT",35,35000.0,Arrays.asList("J2EE","C++","Kafka")));
        employees.add(new Employee(101, "Kamala", "IT",28,25000.0,Arrays.asList("microservices","python")));
        employees.add(new Employee(101, "Anji", "CSE",32,40000.0,Arrays.asList("java","spring")));

        /*
         * 1.Filtering and Collecting: Write a Streams API expression to filter out
         * employees with a salary greater than 50,000 from empList and collect them
         * into a new list.
         */
        List<Employee> empList=employees.stream().filter(e->e.getSalary()>50000).collect(Collectors.toList());
        System.out.println(empList);
        /*
         * 2.Mapping: Transform empList to a list of employee names using the Streams API.
         */
        List<String> collect = employees.stream().map(e->e.getName()).collect(Collectors.toList());
        System.out.println(collect);

        /*
         * 3.Sorting: Sort empList by employee age in ascending order using the Streams
         * API.
         */
        List<String> empAge=employees.stream().sorted(Comparator.comparingInt(Employee::getAge)).map(Employee::getName).collect(Collectors.toList());
        System.out.println("Employee age in Ascending order:"+empAge);


        /*
         *4. Aggregation (Reduction): Use the Streams API to find the total salary of all
         * employees in empList.
         */
        DoubleSummaryStatistics collect2 = employees.stream().collect(Collectors.summarizingDouble(Employee::getSalary));
        System.out.println("Total salary of the employees:"+collect2.getSum());

        /*
         *5. Finding Maximum/Minimum: Write a Streams API expression to find the employee
         * with the highest salary in empList. Group By:
         */

        Optional<Employee> max = employees.stream().max(Comparator.comparingDouble(Employee::getSalary));
        System.out.println("Highest salary Employee name: "+max.get().getName());

        /*
         * 6.Group By: Group employees by department using the Streams API.
         */


        Map<String, List<Employee>> collect3 = employees.stream().collect(Collectors.groupingBy(Employee::getDepartment));
        collect3.forEach((dept1,empList1)->
        {
            System.out.println("department:"+dept1);

            System.out.println(empList1);
        });

        /*
         * Custom Collector: Create a custom collector to partition employees into two
         * groups based on their salary (greater or less than 50,000) using the Streams
         * API.
         *
         */

        Map<Boolean, List<Employee>> collect4 = employees.stream().collect(Collectors.partitioningBy(e->e.getSalary()>50000));

        List<Employee> salaryBelow10000Employees=collect4.get(false);
        System.out.println("Employees list salary less than 50000 "+salaryBelow10000Employees);

        List<Employee> salaryGreaterthan10000Employees=collect4.get(true);
        System.out.println("employees list salary is greater than 50000 "+salaryGreaterthan10000Employees);

        /*
         * Parallel Streams: Convert empList into a parallel stream and demonstrate
         * calculating the count of employees.
         */

        long count = employees.parallelStream().count();
        System.out.println("count of employees from the list"+count);


       /* FlatMap:
        Assuming empList contains a list of employees, each having a list of skills, write a Streams API expression to flatten all skills into a single list.
*/
        List<String> allSkills=employees.stream().flatMap(employee->employee.skills.stream()).collect(Collectors.toList());
        System.out.println("flatten Skills:"+allSkills);

    }
}